package ZMQ::FFI::ContextRole;
$ZMQ::FFI::ContextRole::VERSION = '1.07';
use Moo::Role;

# real underlying zmq ctx pointer
has _ctx => (
    is      => 'rw',
    default => -1,
);

# used to make sure we handle fork situations correctly
has _pid => (
    is      => 'ro',
    default => sub { $$ },
);

has soname => (
    is       => 'ro',
    required => 1,
);

has threads => (
    is        => 'ro',
    predicate => 'has_threads',
);

has max_sockets => (
    is        => 'ro',
    predicate => 'has_max_sockets',
);

requires qw(
    get
    set
    socket
    proxy
    device
    destroy
);

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

ZMQ::FFI::ContextRole

=head1 VERSION

version 1.07

=head1 AUTHOR

Dylan Cali <calid1984@gmail.com>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2015 by Dylan Cali.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
